#!/usr/bin/env python
# needed to run the code as a script, (must make it executable: chmod +x namefile.py)

# Get image from which patches shall be extracted
# Takes input image and crops it around the selected point
# Output: cropped image and resized version of cropped image 
#         the output image is also converted to gray level is the parameter is set
# Example: getinput.py ~/Downloads/alley.png 200 200 100 0 5 test1.png test1Z.png

import sys
import PIL.Image
from lib import image



def process_input(namein, x0, y0, winsize, usegray, resizefactor, nameout1, nameout2):

        img = image(namein)
        #Subimage corners
        x1=x0-winsize/2
        if x1 < 0:
            x1=0
        y1=y0-winsize/2
        if y1 < 0:
            y1=0
        x2=x0+winsize/2
        if x2 >= img.size[0]:
            x2=img.size[0]-1
        y2=y0+winsize/2
        if y2 >= img.size[1]:
            y2=img.size[1]-1

        img.crop((x1, y1, x2, y2))

        #coordinates of the selected pixel in the new subimage
        x0b=x0-x1
        y0b=y0-y1
        print('%i %i'%(x0b, y0b))
            
        #convert to gray image, if needed
        if usegray == 1 and img.im.mode == "RGB":
            # compute grey level image: I = (R + G + B) / 3
            rgb2I = (0.333333, 0.333333, 0.333333, 0,
                     0, 0, 0, 0,
                     0, 0, 0, 0 )
            imgray = img.im.convert("L", rgb2I)
            imgray.save(nameout1)
        else:
            img.save(nameout1)

        #displayable image: resized x resizefactor, center marked with red dot
        img = image(nameout1)
        cx=x0b
        cy=y0b
        img.resize(float(resizefactor), method="pixeldup")
        imDisp = PIL.Image.new("RGB", img.size)
        pix = img.im.load() 
        pixDisp = imDisp.load()
        for y in range(0, img.size[1]):
            for x in range(0, img.size[0]):
                if img.im.mode == "RGB":
                    pixDisp[x, y] = pix[x, y]
                else:
                    pixDisp[x, y] = (pix[x, y], pix[x, y], pix[x, y])

        pixDisp[resizefactor*cx+resizefactor/2, resizefactor*cy+resizefactor/2] = (255, 0 , 0)
        pixDisp[resizefactor*cx+resizefactor/2-1, resizefactor*cy+resizefactor/2] = (255, 0 , 0)
        pixDisp[resizefactor*cx+resizefactor/2+1, resizefactor*cy+resizefactor/2] = (255, 0 , 0)
        pixDisp[resizefactor*cx+resizefactor/2, resizefactor*cy+resizefactor/2-1] = (255, 0 , 0)
        pixDisp[resizefactor*cx+resizefactor/2, resizefactor*cy+resizefactor/2+1] = (255, 0 , 0)
        
        imDisp.save(nameout2)




if __name__ == "__main__":
    argc = len(sys.argv)
    if argc <= 1:
#        print "Usage: getinput input x0 y0 size usegray(0/1) resizefactor output1 output2" #python 2 format
        print('Usage: getinput input x0 y0 size usegray(0/1) resizefactor output1 output2')
        sys.exit(0)

    namein = sys.argv[1]
    x0 = int(round(float(sys.argv[2])))
    y0 = int(round(float(sys.argv[3])))
    size = int(sys.argv[4])
    usegray = int(sys.argv[5])
    resizefactor = int(sys.argv[6])
    nameout1 = sys.argv[7]
    nameout2 = sys.argv[8]

    process_input(namein, x0, y0, size, usegray, resizefactor, nameout1, nameout2)

    


