#!/usr/bin/env python
# needed to run the code as a script, (must make it executable: chmod +x namefile.py)

# Get projections (of patches in 3D space) histograms 
# Compute Anderson-Darling normality test for each histogram


import sys
import PIL.Image
from lib import image
import numpy
import math
import scipy.stats as si

def max_qhistograms(im):
    h0 = im.im.histogram()
    hR = h0[0:256]
    hG = h0[256:512]
    hB = h0[512:768]
    #quantize histograms
    hqR=numpy.zeros(16)
    hqG=numpy.zeros(16)
    hqB=numpy.zeros(16)

    q=16
    for x in range(0, 256):
#      hqR[x/q]=hqR[x/q]+hR[x] #python 2 format for integer division
#      hqG[x/q]=hqG[x/q]+hG[x] #python 2 format for integer division
#      hqB[x/q]=hqB[x/q]+hB[x] #python 2 format for integer division
      hqR[x//q]=hqR[x//q]+hR[x]
      hqG[x//q]=hqG[x//q]+hG[x]
      hqB[x//q]=hqB[x//q]+hB[x]

    return max(max(hqR), max(hqG), max(hqB))



def histogramView(im, imH, option, maxRef):
  """
    draws in imH the histogram values stored of im[option]
  """
  offsetH = {"R":0, "G":256, "B":512}
  color = {"R":(255, 0, 0), 
           "G":(0, 255, 0), 
           "B":(0, 0, 255)}

  h0 = im.im.histogram()

  h = h0[offsetH[option]:offsetH[option]+256]
  
  sizeH = imH.size
  ymax = sizeH[1]-1
  ymin = 0
  scaleH = float(sizeH[1]-1)/float(maxRef)
  
  draw = PIL.ImageDraw.Draw(imH)
  
  #statistics
  S = sum(h)
  mean = 0
  for x in range(0, 256):
    mean+=float(x*h[x])
  mean/=float(S)
  std = 0
  for x in range(0, 256):
    std+=float((x-mean)*(x-mean)*h[x])
  std/=float(S)
  std=math.sqrt(std)

  K=1/math.sqrt(2*math.pi)


  #quantize histogram (16 bins of size=16)
  hq=numpy.zeros(16)

  q=16
  for x in range(0, 256):
#    hq[x/q]=hq[x/q]+h[x] #python 2 format for integer division
    hq[x//q]=hq[x//q]+h[x]


  #copy values to h for display
  for x in range(0, 256):
#    h[x]=hq[x/q] #python 2 format for integer division
    h[x]=hq[x//q]

  S = sum(h)  
  Sz = 0
  for x in range(0, 256):
    #reference values (Gaussian)
    pz=K*math.exp(-(x-mean)*(x-mean)/(2*std*std))/std
    z=pz*S
    Sz+=z

  #print "S=%f  Sz=%f\n"%(S, Sz)

  Szz=0
  for x in range(0, 256):
    #image values
    if (ymax-int(h[x] * scaleH)) < ymin:
      #saturated value
      draw.line([(x, ymax), 
                 (x, ymin)],
                fill=0)
    else:
      draw.line([(x, ymax), 
                 (x, ymax-int(h[x] * scaleH))],
                fill=color[option])
    #reference values (Gaussian)
    pz=K*math.exp(-(x-mean)*(x-mean)/(2*std*std))/std
    z=pz*S*(S/Sz)
    Szz+=z
    if (ymax-int(z * scaleH)) >= ymin:
      draw.point([(x, ymax-int(z * scaleH))],
                fill=0)

  #print "S=%f  Sz=%f  Szz=%f\n"%(S, Sz, Szz)

  del draw

  #Compute Anderson-Darling normality test
  #Save p-value to text file
  #References:
  #https://stats.stackexchange.com/questions/350443/how-do-i-get-the-p-value-of-ad-test-using-the-results-of-scipy-stats-anderson
  #https://www.spcforexcel.com/knowledge/basic-statistics/anderson-darling-test-for-normality
  channel = {"R":0, "G":1, "B":2}
  datapix=list(im.im.getdata())   
  data=[x[channel[option]] for x in datapix] 

  AD, crit, sig = si.anderson(data, dist='norm')
  n=len(data)
  #Adjusted AD
  AD = AD*(1 + (.75/n) + 2.25/(n**2))
  if AD >= .6:
    p = math.exp(1.2937 - 5.709*AD - .0186*(AD**2))
  elif AD >=.34:
    p = math.exp(.9177 - 4.279*AD - 1.38*(AD**2))
  elif AD >.2:
    p = 1 - math.exp(-8.318 + 42.796*AD - 59.938*(AD**2))
  else:
    p = 1 - math.exp(-13.436 + 101.14*AD - 223.73*(AD**2))
    
  print("pvalue=%2.4f"%(p))
  
  file_p = open("pvalue"+option+".txt","w")
  file_p.write("%2.4f\n"%(p))
  file_p.close()


#Main code

im = image('viewsPCA_RGB.png')

maxH = max_qhistograms(im)

imH = PIL.Image.new('RGB', (256, 128), (255, 255, 255))
histogramView(im, imH, option="R", maxRef=maxH)
imH.save('histPCA_R.png')

imH2 = PIL.Image.new('RGB', (256, 128), (255, 255, 255))
histogramView(im, imH2, option="G", maxRef=maxH)
imH2.save('histPCA_G.png')

imH3 = PIL.Image.new('RGB', (256, 128), (255, 255, 255))
histogramView(im, imH3, option="B", maxRef=maxH)
imH3.save('histPCA_B.png')



