/*
 * Copyright (c) 2019 Jose-Luis Lisani <joseluis.lisani@uib.es>
 * All rights reserved.
 *
 * Code inspired in Michael Chen Matlab code 
 * http://www.mathworks.com/matlabcentral/fileexchange/26184-em-algorithm-for-gaussian-mixture-model

 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
 
#ifndef GAUSSIANMIX_H
#define GAUSSIANMIX_H

//compute Gaussian Mixture (GM) for a set of vectors
void GaussianMixture(const double **dataIn, const int ndataIn, const int dimIn, 
                     const int ngaussIn, int &ngaussOut, int **labelsOut, 
                     double ***meanOut, double ****covarOut,
                     double **probOut);

//Compute probnk=prob(gaussian k | given vector n)
double **probnk_GaussianMixture(const double **dataIn, const int ndataIn, 
                                const int dimIn, const int ngaussIn,
                           double **meanIn, double ***covarIn, double *probIn);

//For each input vector, compute labels: assign Gaussian with highest probnk
//where probnk=prob(gaussian k | given vector n)
int *labelsGaussianMixture(const double **dataIn, const int ndataIn, 
                           const int dimIn, const int ngaussIn,
		     	           double **meanIn, double ***covarIn, double *probIn);

#endif
