/*
 * Copyright (c) 2011 Jose-Luis Lisani <joseluis.lisani@uib.es>
 * All rights reserved.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef GRADIENTS_LIB_H
#define GRADIENTS_LIB_H

#define nearest(x) (ceil(x)-x < x-floor(x))?(ceil(x)):(floor(x))


//data structure storing gradient information
struct gradient_info {
  int x, y; //pixel location for which gradient vector is computed
  double gx, gy, gmod2; //gradient vector and its magnitude
};

//compute gradients for an input image
void compute_gradient(float **Gx_out, float **Gy_out, unsigned char *im, 
                      int w, int h);

//compute gradients for an input image and store information in data structure
struct gradient_info *get_gradient_info(unsigned char *im, int w, int h, 
                                        const char *methodGrad);

//Compute smooth gradient values
//Gradient vector computed from the neighborhood (radius: rad) of a pixel 
//given an initial estimate of the gradients
struct gradient_info *compute_gradient_smooth(struct gradient_info *gdata0, unsigned char *im, int w, int h, int r);

//Obtain Canny extrema of an image
struct gradient_info *get_canny_extrema(struct gradient_info *gdata, int w, int h, float th, int &nextrema);

//Obtain Harris extrema of an image
struct gradient_info *get_Harris_extrema(struct gradient_info *gdata, int w, int h, 
											float thk, float thT, int &nextrema);
								 


#endif


