/*
 * Copyright (c) 2011 Jose-Luis Lisani <joseluis.lisani@uib.es>
 * All rights reserved.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef PATCHES_EXTRACTION_LIB_H
#define PATCHES_EXTRACTION_LIB_H

//data structure that stores patches information
struct patch_info {
  float cx, cy; //location of the center of the patch
  float vx, vy; //orientation of the patch
  double *patch1, *patch2, *patch3; //patch data
  float a, b; //intensity normalization parameters
};



//gets patches centered at a set of selected pixels
double **get_selected_patches(unsigned char *R, unsigned char *G, 
                              unsigned char *B, int w, int h,
						      struct patch_info *info, int ninfo, 
						      int wpatch, int hpatch,
						      unsigned char use_orientation,
						      unsigned char use_normMean,
							  unsigned char use_normVar, 
							  int &dim, int &npatches, const char *methodInterp,
                              int npatchesmax);
                              
                              
//allocates data structures to store data of patches centered at random pixels
struct patch_info *select_patches_centers_random(int w, int h, 
                                                 int wpatch, int hpatch, 
                                                 float prcpatches,
												 int &npatches, float offset);
												 
//allocats data structures to store data of patches at selected extrema
struct patch_info *select_patches_centers_GradExtrema(struct gradient_info *gradE, 
                                                      int ngradE, int &npatches, 
                                                      float offset);
				
//attaches gradient information to patches data structures
void add_orientation_info(struct patch_info *info, int ninfo, 
                          struct gradient_info *gradI, 
						  int w, float th);

//gets closest patches (L2 distance) to patch centered (aproximately) at (x0, y0) 
double **select_closest(double **patchesIn, int npatchesIn, 
                        struct patch_info *infoIn, int ninfoIn,
                        int dim, int x0, int y0, int nclosest, float dmax,
                        int &npatchesOut, struct patch_info **infoOut, 
                        int &id0);							  
							  
//store in file integer values indicating some information about the stored data
void writeDdatalabel(double *data, int size, int label, const char *name);

//writes patches data to file
void write2Ddata(double **data, int n, int dim, const char *name);

//reads patches data from file
double **read2Ddata(const char *name, int &n, int &dim);

//reads from file integer values indicating some information about the data
double *readDdatalabel(const char *name, int &size);

//reads patches label from file
int readlabel(const char *name);

//saves patches additional information (location, gradient, etc) to binary file
void save_patch_info(struct patch_info *info, int ninfo, int w, int h, 
                     int wpatch, int hpatch, 
                     unsigned char use_orientation, 
                     const char *name);
                     
//reads patches additional information (location, gradient,etc) from binary file
struct patch_info *read_patch_info(const char *name, int &ninfo, int &w, int &h, 
                                   int &wpatch, int &hpatch, 
                                   unsigned char &use_orientation);
    
//delete information of patches
void delete_patch_info(struct patch_info *info, int ninfo);

//synthesizes 8 patches using a linear combination of basis vectors
double **simpatches(double *mean, double *val, double **vec, int dim);

                           
						  
						  

#endif



