#!/bin/sh
set -e

echo "run.sh"
echo "Parameters:"
echo "x0="$1 " y0="$2 " size="$3 " usegray="$4 " spatch="$5 " extraction="$6 
echo "norm. orientation="$7 " norm. mean="$8 " norm. variance="$9
echo "nclosest="${10}  " ngauss="${11}    
echo "pca1="${12}  " pca2="${13}  " pca3="${14}

#Example:
# ./run.sh 200 200 100 0 8 random 1 0 0 20 2 1 2 3


#STEP 1. Process input image and get subwindow centered at selected pixel
#STEP 2. Extract all patches in selected subwindow


x0=$1
y0=$2
winsize=$3
usegray=$4
spatch=$5
extraction=$6
orient=$7
normMean=$8
normVar=$9

script_extract.sh input_0.png $x0 $y0 $winsize $usegray $spatch $extraction \
                  $orient $normMean $normVar

#Check if patches have been found
if [ ! -f "patches.dat" ]; then
    exit 0
fi

#STEP 3. Get closest patches
#STEP 4. PCA and GM analysis

nclosest=${10}
ngauss=${11}

script_closest.sh $nclosest $ngauss

#STEP 5. Get 3D projections of patches

pca1=${12}
pca2=${13}
pca3=${14}

script_3dprojections.sh $pca1 $pca2 $pca3



#STEP 6. Save relevant information

#Save parameters info

echo "x0="$x0 > algo_info.txt
echo "y0="$y0 >> algo_info.txt
echo "winsize="$winsize >> algo_info.txt
echo "usegray="$usegray >> algo_info.txt
echo "spatch="$spatch >> algo_info.txt
echo "extraction="$extraction >> algo_info.txt
echo "orient="$orient >> algo_info.txt
echo "normMean="$normMean >> algo_info.txt
echo "normVar="$normVar >> algo_info.txt
echo "nclosest="$nclosest >> algo_info.txt
echo "ngauss="$ngauss >> algo_info.txt
echo "pca1="$pca1 >> algo_info.txt
echo "pca2="$pca2 >> algo_info.txt
echo "pca3="$pca3 >> algo_info.txt



#Get additional info 
nclosestf=`cat closestinfo.txt | cut -d' ' -f3`
echo "nclosestf="$nclosestf >> algo_info.txt


stdpca=`cat pcastd.txt | cut -d' ' -f2`
echo "stdpca="$stdpca >> algo_info.txt

n=$((ngauss-1))
for i in $(seq 0 $n); do 
    std=`cat gaussstd.txt | grep ${i}: | cut -d' ' -f3`
    echo "stdgm_"${i}"="$std >> algo_info.txt
done
for i in $(seq 0 $n); do 
    std=`cat gaussprobs.txt | grep ${i}: | cut -d' ' -f3`
    echo "probgm_"${i}"="$std >> algo_info.txt
done

pvalue1=`cat pvalueR.txt`
pvalue2=`cat pvalueG.txt`
pvalue3=`cat pvalueB.txt`
echo "pvalue1="$pvalue1 >> algo_info.txt
echo "pvalue2="$pvalue2 >> algo_info.txt
echo "pvalue3="$pvalue3 >> algo_info.txt


if [ -f "gaussprobs.txt" ]; then
    echo "gmerror= " >> algo_info.txt
else
    echo "gmerror=Not enough patches to compute the Gaussian Mixture" >> algo_info.txt
fi

if [ -f "gmwarning.txt" ]; then
    echo "gmwarning=Warning: for the Gaussian Mixture the number of selected patches may have been insufficient to reach a reliable result" >> algo_info.txt
else
    echo "gmwarning= " >> algo_info.txt
fi







