#!/bin/sh
set -e

# Extract closest patches to the patch centered at a given pixel
# Analyse the set of extracted patches (PCA and GM)

echo "script_closest"
echo "Parameters:"
echo "nclosest="$1 " ngauss="$2

#Example:
# script_closest.sh 20 2


#resize factor for images display
resizefactor=5

#STEP 3. Get closest patches

nclosest=$1
x0=`cat pointcoord.txt | cut -d' ' -f1`
y0=`cat pointcoord.txt | cut -d' ' -f2`

patches_closest patches patchesClosest $x0 $y0 $nclosest 

#closest to closest
#patchesC2C_0_C.png == patchesClosestC.png (Image of centers of closest patches)
patches_closest2closest patches patchesC2C $x0 $y0 $nclosest \
                        -o input_0_sel.png -z $resizefactor
cp patchesC2C_0_C.png patchesClosestC.png
                          
#STEP 4. Analyse extracted patches (PCA and GM)

ngauss=$2

#PCA
patches_pca patchesClosest.dat basis_pca 

#GM
patches_gm patchesClosest.dat basis_gaussiansmixture $ngauss


#Generate simulated patches
#normalize simulated patches with mean=128 and std=40, for visualization

#From PCA
patches_sim basis_pca.dat patchesSim_pca -m 128 -s 40

#From GM
patches_sim basis_gaussiansmixture.dat patchesSim_gaussiansmixture -m 128 -s 40


#Resize basis images, if they are small
#width must be at least 300 pixels
wref=300

resize_basis_pca.py $wref

resize_basis_gm.py $wref $ngauss


#rename basis_gaussiansmixture_PCA_EigV.png to basis_pca_Eigv.png
#CHECK IF THIS FILE IS CREATED 
file="basis_pca_EigV.png"
if [ -f "$file" ]; then
    cp basis_pca_EigV.png basis_pca_Eigv_orig.png 
fi
file="basis_gaussiansmixture_PCA_EigV.png"
if [ -f "$file" ]; then             
    mv basis_gaussiansmixture_PCA_EigV.png basis_pca_EigV.png
fi


#display with different color the patches associated to each Gaussian 
#in the mixture
patches_gmprobs patchesClosest.dat patchesClosest_loc.dat \
                basis_gaussiansmixture.dat recons_gaussiansmixture \
                -z $resizefactor -c -o input_0_sel.png


#Top 10 closest patches
resizeimage.py patchesClosest_ref.png $resizefactor
resizeimage.py patchesClosest_top10.png $resizefactor
resizeimage.py patchesClosest_all.png $resizefactor

n=$((ngauss-1))
for i in $(seq 0 $n); do 
    resizeimage.py recons_gaussiansmixture_top10_${i}.png $resizefactor 
done



